"""Test the validators module."""

# Copyright (C) 2021-2024  Blue Brain Project, EPFL
#
# SPDX-License-Identifier: Apache-2.0

import pytest
from jsonschema.exceptions import ValidationError

from diameter_synthesis import validators


def test_defaults(config, model_params):
    """Test the validation of JSON schemas."""
    validators.validate_model_params(model_params)
    validators.validate_config(config)

    config["features"] = {"FEATURE 1": {"apical_dendrite": 0.2}}
    validators.validate_config(config)

    config["features"] = {"FEATURE 2": {"UNKNOWN": 0.2}}
    with pytest.raises(ValidationError):
        validators.validate_config(config)



"""Diameter-synthesis expceptions."""

# Copyright (C) 2021-2024  Blue Brain Project, EPFL
#
# SPDX-License-Identifier: Apache-2.0


class DiameterSynthesisError(Exception):
    """Base diameter-synthesis exception."""



"""Test the build_models module."""

# Copyright (C) 2021-2024  Blue Brain Project, EPFL
#
# SPDX-License-Identifier: Apache-2.0

import morphio.mut
import neurom as nm
from morphio import SectionType

from diameter_synthesis import build_diameters
from diameter_synthesis import build_models

from .testing_tools import _compare_diameters
from .testing_tools import compare_dicts


def test_build_model(single_pop, simpler_model_params, simpler_model_data):
    """Test the build function."""
    # Test with generic model
    config = {
        "models": ["simpler"],
        "neurite_types": ["basal_dendrite", "apical_dendrite"],
    }
    res = build_models.build(single_pop, config, with_data=True)
    res_models_params = build_models.build(single_pop, config, with_data=False)

    assert len(res) == 2
    assert compare_dicts(res, [simpler_model_params, simpler_model_data], precision=3)
    assert compare_dicts(res_models_params, simpler_model_params, precision=3)


def test_build_model_missing_neurite_type(tmpdir, neuron, simpler_model_params, simpler_model_data):
    """Test the build function with missing neurites of given types."""
    # Remove axon from neuron
    for sec in neuron.root_sections:
        if sec.type == SectionType.axon:
            neuron.delete_section(sec)
    neuron_path = tmpdir / "neuron.swc"
    neuron.write(neuron_path)
    single_pop = [nm.load_morphology(neuron_path)]

    # Test with generic model
    config = {
        "models": ["simpler"],
        "neurite_types": ["basal_dendrite", "axon"],
    }
    res = build_models.build(single_pop, config, with_data=True)
    res_models_params = build_models.build(single_pop, config, with_data=False)

    # Build expected results
    simpler_model_params["axon"] = []
    del simpler_model_params["apical_dendrite"]
    for i in simpler_model_data:
        i["axon"] = []
        del i["apical_dendrite"]
    simpler_model_data[2]["axon"] = None

    # Check results
    assert len(res) == 2
    assert compare_dicts(res, [simpler_model_params, simpler_model_data], precision=3)
    assert compare_dicts(res_models_params, simpler_model_params, precision=3)


def test_build_diameters(simpler_config, simpler_model_params, small_morph, test_data_path):
    """Test the main build function to diametrize a neuron."""
    neurite_types = ["basal_dendrite", "apical_dendrite"]

    build_diameters.build(small_morph, neurite_types, simpler_model_params, simpler_config)
    expected = morphio.mut.Morphology(test_data_path / "simpler_morph_diametrized.asc")
    _compare_diameters(expected, small_morph)



"""diameter-synthesis package.

Diametrize cells.
"""

# Copyright (C) 2021-2024  Blue Brain Project, EPFL
#
# SPDX-License-Identifier: Apache-2.0

import importlib.metadata

__version__ = importlib.metadata.version("diameter-synthesis")



"""Setup for the diameter-synthesis package."""

# Copyright (C) 2021-2024  Blue Brain Project, EPFL
#
# SPDX-License-Identifier: Apache-2.0

from pathlib import Path

from setuptools import find_namespace_packages
from setuptools import setup

reqs = [
    "click>=7.0",
    "jsonschema>=3",
    "matplotlib>=3.4",
    "morphio>=3.3.4",
    "neurom>=3.0,<4.0",
    "numpy>=1.22.0",
    "pandas>=1.1",
    "scipy>=1.6",
    "tqdm>=4.50",
]

doc_reqs = [
    "docutils<0.21",
    "m2r2",
    "sphinx",
    "sphinx-bluebrain-theme",
    "sphinx-jsonschema",
    "sphinx_click",
]

test_reqs = [
    "diff_pdf_visually>=1.7.0",
    "decorator>=4",
    "mock>=3",
    "pytest>=6",
    "pytest-click>=1",
    "pytest-console-scripts>=1.3",
    "pytest-cov>=3",
    "pytest-html>=2",
    "pytest-xdist>=2",
]

setup(
    name="diameter-synthesis",
    author="Blue Brain Project, EPFL",
    description="Diametrize cells.",
    long_description=Path("README.md").read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",
    url="https://diameter-synthesis.readthedocs.io",
    project_urls={
        "Tracker": "https://github.com/BlueBrain/diameter-synthesis/issues",
        "Source": "https://github.com/BlueBrain/diameter-synthesis",
    },
    license="Apache License 2.0",
    packages=find_namespace_packages(include=["diameter_synthesis*"]),
    python_requires=">=3.8",
    use_scm_version=True,
    setup_requires=[
        "setuptools_scm",
    ],
    install_requires=reqs,
    extras_require={
        "docs": doc_reqs,
        "plot": "seaborn>=0.13",
        "test": test_reqs,
    },
    entry_points={
        "console_scripts": [
            "diameter-synthesis=diameter_synthesis.cli:main",
        ],
    },
    include_package_data=True,
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
)
